@@ -62,15 +62,16 @@ def Downsample(dim, dim_out = None):
     )
 
 class LayerNorm(nn.Module):
-    def __init__(self, dim):
+    def __init__(self, dim, bias = False):
         super().__init__()
         self.g = nn.Parameter(torch.ones(1, dim, 1, 1))
+        self.b = nn.Parameter(torch.zeros(1, dim, 1, 1)) if bias else None
 
     def forward(self, x):
         eps = 1e-5 if x.dtype == torch.float32 else 1e-3
         var = torch.var(x, dim = 1, unbiased = False, keepdim = True)
         mean = torch.mean(x, dim = 1, keepdim = True)
-        return (x - mean) * (var + eps).rsqrt() * self.g
+        return (x - mean) * (var + eps).rsqrt() * self.g + default(self.b, 0)
 
 class SinusoidalPosEmb(nn.Module):
     def __init__(self, dim):
@@ -195,6 +196,19 @@ class Attention(nn.Module):
         out = rearrange(out, 'b h (x y) d -> b (h d) x y', x = h, y = w)
         return self.to_out(out)
 
+# conditioning class
+
+class FourierConditioning(nn.Module):
+    def __init__(self, dim):
+        super().__init__()
+        self.norm_input = LayerNorm(dim, bias = True)
+        self.norm_condition = LayerNorm(dim, bias = True)
+
+    def forward(self, x, c):
+        normed_x = self.norm_input(x)
+        normed_c = self.norm_condition(c)
+        return (normed_x * normed_c) * c   # eq 3 in paper
+
 # model
 
 class Unet(nn.Module):
@@ -207,7 +221,7 @@ class Unet(nn.Module):
         channels = 3,
         self_condition = False,
         resnet_block_groups = 8,
-        learned_variance = False
+        conditioning_klass = FourierConditioning
     ):
         super().__init__()
 
@@ -242,6 +256,8 @@ class Unet(nn.Module):
 
         num_resolutions = len(in_out)
 
+        self.conditioners = nn.ModuleList([])
+
         # downsampling encoding blocks
 
         self.downs = nn.ModuleList([])
@@ -249,11 +265,12 @@ class Unet(nn.Module):
         for ind, (dim_in, dim_out) in enumerate(in_out):
             is_last = ind >= (num_resolutions - 1)
 
+            self.conditioners.append(conditioning_klass(dim_in))
+
             self.downs.append(nn.ModuleList([
                 block_klass(dim_in, dim_in, time_emb_dim = time_dim),
                 block_klass(dim_in, dim_in, time_emb_dim = time_dim),
                 Residual(LinearAttention(dim_in)),
-                LayerNorm(dim_in),
                 Downsample(dim_in, dim_out) if not is_last else nn.Conv2d(dim_in, dim_out, 3, padding = 1)
             ]))
 
@@ -285,11 +302,8 @@ class Unet(nn.Module):
 
         # projection out to predictions
 
-        default_out_dim = channels * (1 if not learned_variance else 2)
-        self.out_dim = default(out_dim, default_out_dim)
-
         self.final_res_block = block_klass(dim * 2, dim, time_emb_dim = time_dim)
-        self.final_conv = nn.Conv2d(dim, self.out_dim, 1)
+        self.final_conv = nn.Conv2d(dim, channels, 1)
 
     def forward(
         self,
@@ -313,7 +327,7 @@ class Unet(nn.Module):
 
         h = []
 
-        for (block1, block2, attn, norm, downsample), (cond_block1, cond_block2, cond_attn, cond_norm, cond_downsample) in zip(self.downs, self.cond_downs):
+        for (block1, block2, attn, downsample), (cond_block1, cond_block2, cond_attn, cond_downsample), conditioner in zip(self.downs, self.cond_downs, self.conditioners):
             x = block1(x, t)
             c = cond_block1(c, t)
 
@@ -325,23 +339,10 @@ class Unet(nn.Module):
             x = attn(x)
             c = cond_attn(c)
 
-            # they create an attentive map A by element-wise multiplication of 
-            # then they use it to modulate it to modulate the condition in fourier space (ff-parse)
+            # condition using modulation of fourier frequencies with attentive map
+            # you can test your own conditioners by passing in a different conditioner_klass , if you believe you can best the paper
 
-            normed_x, normed_c = norm(x), cond_norm(c)
-
-            A = normed_x * normed_c * c
-
-            # fc stands for conditioning in fourier space
-
-            fc = fft2(c)
-
-            fc = fc * A
-
-            c = ifft2(fc).real
-            c = c.type(dtype)
-
-            # </conditioning>
+            c = conditioner(x, c)
 
             h.append(x)
 
